/*------------------------------------------------------------------------
 |  FILE            HeaderLabel.java
 |  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 +----------------------------------------------------------------------*/

package no.shhsoft.awt;

import java.awt.*;

/**
 * A label suitable for use as a title for panels. Has a gray background,
 * and "almost white" foreground. Mimics the headers used in eg.
 * Microsoft Outlook.
 *
 * @author    Sverre H. Huseby &lt;<A HREF="mailto:shh@thathost.com">
 *                                 shh@thathost.com</A>&gt;
 * @version   $Id: HeaderLabel.java,v 1.1 2000/09/20 19:39:24 sverrehu Exp $
 */
public class HeaderLabel
extends Label {
/*-----------------------------------------------------------------------+
 |  PRIVATE PART                                                         |
 +----------------------------------------------------------------------*/
    private void init() {
        setBackground(Color.gray);
        setForeground(Color.lightGray.brighter().brighter());

        /* choose a font that is bigger than the default font. we
         * don't want to hardcode the actual size, so we request the
         * normal size first, and add some points. */
        setFont(new Font("SansSerif", Font.BOLD, 16));
    }

/*-----------------------------------------------------------------------+
 |  PUBLIC INTERFACE                                                     |
 +----------------------------------------------------------------------*/
    /**
     * Constructs a new <CODE>HeaderLabel</CODE> with no text in it. Use
     * <CODE>setText</CODE> to set the wanted text later.
     */
    public HeaderLabel() {
        super();
        init();
    }

    /**
     * Constructs a new <CODE>HeaderLabel</CODE> with the given text.
     *
     * @param      text    the text to show in the label.
     */
    public HeaderLabel(String text) {
        super(" " + text);
        init();
    }

    public void setText(String text) {
        super.setText(" " + text);
    }
}
