/*------------------------------------------------------------------------
 |  FILE            LoginDialog.java
 |  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 +----------------------------------------------------------------------*/

package no.shhsoft.awt;

import java.awt.*;
import java.awt.event.*;

/**
 *
 * @author    Sverre H. Huseby &lt;<A HREF="mailto:shh@thathost.com">
 *                                 shh@thathost.com</A>&gt;
 * @version   $Id: LoginDialog.java,v 1.1 2000/09/20 19:39:24 sverrehu Exp $
 */
public final class LoginDialog
extends ModalButtonDialog {
/*-----------------------------------------------------------------------+
 |  PRIVATE PART                                                         |
 +----------------------------------------------------------------------*/
    private GridBagLayout      gbl;
    private GridBagConstraints gbc;
    private Panel              panel;
    private String             usernameLabel = "User Name";
    private String             passwordLabel = "Password";
    private TextField          username;
    private TextField          password;

    private void addField(Component field,
                          int gwidth, int gheight,
                          int gx, int wx, int wy) {
        if (gx == 0 && wy == 0) /* label */
            gbc.anchor = GridBagConstraints.EAST;
        else
            gbc.anchor = GridBagConstraints.WEST;
        gbc.gridwidth = gwidth;
        gbc.gridheight = gheight;
        gbc.gridx = gx;
        gbc.weightx = wx;
        gbc.weighty = wy;
        gbl.setConstraints(field, gbc);
        panel.add(field);
    }

    private void addLabelAndFieldLine(String labelText, Component field) {
        addField(new Label(labelText), 1, 1, 0, 0, 0);
        addField(field,
                 GridBagConstraints.REMAINDER, 1,
                 GridBagConstraints.RELATIVE, 1, 0);
    }

    private Panel setupPanel() {
        panel = new Panel();
        gbl = new GridBagLayout();
        gbc = new GridBagConstraints();
        panel.setLayout(gbl);
        gbc.insets = new Insets(2, 5, 2, 5);
        gbc.anchor = GridBagConstraints.WEST;

        username = new TextField(15);
        addLabelAndFieldLine(usernameLabel, username);
        password = new TextField(15);
        password.setEchoChar('*');
        addLabelAndFieldLine(passwordLabel, password);

        return panel;
    }

/*-----------------------------------------------------------------------+
 |  PUBLIC INTERFACE                                                     |
 +----------------------------------------------------------------------*/
    public LoginDialog(Component parent, String title,
                       String usernameLabel, String passwordLabel) {
        super(parent, title, OK | CANCEL, OK);
        if (usernameLabel != null)
            this.usernameLabel = usernameLabel;
        if (passwordLabel != null)
            this.passwordLabel = passwordLabel;
        setCenterComponent(setupPanel());
        setVisible(true);  /* will block, as this box is modal */
    }

    public LoginDialog(Component parent, String title) {
        this(parent, title, null, null);
    }

    public String getUserName() {
        if (username == null || getButtonCode() == CANCEL)
            return null;
        return username.getText().trim();
    }

    public String getPassword() {
        if (password == null || getButtonCode() == CANCEL)
            return null;
        return password.getText();
    }
}
