/* -*- mode: Java; c-basic-offset: 4 -*- */
/*------------------------------------------------------------------------
 |  FILE            PropertyEditor.java
 |  PART OF         WebWatcher - A web-page update monitor
 |  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 +----------------------------------------------------------------------*/

import java.io.*;
import java.awt.*;
import java.awt.event.*;

import no.shhsoft.awt.*;

/**
 *
 *
 * @author   Sverre H. Huseby
 *           &lt;<A HREF="mailto:shh@thathost.com">shh@thathost.com</A>&gt;
 * @version  $Id: PropertyEditor.java,v 1.1.1.1 2000/10/03 12:41:14 lemire Exp $
 */
public class PropertyEditor implements ItemListener {
/*-----------------------------------------------------------------------+
 |  PRIVATE PART                                                         |
 +----------------------------------------------------------------------*/
    private PropertyProvider prop;
    private boolean          okPressed;

    private Label     labelStatusFile  = new Label("Status file");
    private TextField textStatusFile   = new TextField(60);
    private Label     labelBrowser     = new Label("Web browser (e.g. netscape remote openurl(%u,new_window))");
    private TextField textBrowser      = new TextField(60);
    private Label     labelAlternateBrowser = new Label("Fail-safe Web browser (e.g. netscape %u)");
    private TextField textAlternateBrowser = new TextField(60);
    private Checkbox  boxNewRefresh    = new Checkbox("Refresh new pages");
    private Checkbox  boxAutoRefresh   = new Checkbox("Refresh every");
    private TextField textAutoRefresh  = new TextField(4);
    private Label     labelAutoRefresh = new Label("minutes");
    private Checkbox  boxDoSort        = new Checkbox("Sort list");
    private Checkbox  boxNewOnTop      = new Checkbox("Updated pages on top");
    private Checkbox  boxIgnoreCaseInSort   = new Checkbox("Ignore case when sorting");
    private Checkbox  boxUnderline     = new Checkbox("Underline links");
    private Checkbox  boxUseProxy      = new Checkbox("Use HTTP proxy");
    private Label     labelProxyHost   = new Label("host");
    private TextField textProxyHost    = new TextField(20);
    private Label     labelProxyPort   = new Label("port");
    private TextField textProxyPort    = new TextField(5);


    private void addField(Panel panel,
                          GridBagLayout gbl, GridBagConstraints gbc,
                          Component field, int gwidth, int gx, int wx) {
        gbc.gridwidth = gwidth;
        gbc.gridx = gx;
        gbc.weightx = wx;
        gbc.fill = (wx > 0) ? GridBagConstraints.HORIZONTAL
                            : GridBagConstraints.NONE;
        gbl.setConstraints(field, gbc);
        panel.add(field);
    }

    private Panel setupPanel() {
        Panel              panel, proxyPanel;
        GridBagLayout      gbl;
        GridBagConstraints gbc;

        panel = new Panel();

        gbl = new GridBagLayout();
        gbc = new GridBagConstraints();
        panel.setLayout(gbl);
        gbc.insets = new Insets(3, 10, 3, 10);
        gbc.anchor = GridBagConstraints.WEST;

        textStatusFile.setText(prop.getProperty("pagesFile"));
        addField(panel, gbl, gbc, labelStatusFile, 1, 0, 0);
        addField(panel, gbl, gbc, textStatusFile,
                 GridBagConstraints.REMAINDER,
                 GridBagConstraints.RELATIVE, 1);

        textBrowser.setText(prop.getProperty("browser"));
        textAlternateBrowser.setText(prop.getProperty("alternatebrowsercommand"));
        addField(panel, gbl, gbc, labelBrowser, 1, 0, 0);
        addField(panel, gbl, gbc, textBrowser,
                 GridBagConstraints.REMAINDER,
                 GridBagConstraints.RELATIVE, 1);
        addField(panel, gbl, gbc, labelAlternateBrowser, 1, 0, 0);
        addField(panel, gbl, gbc, textAlternateBrowser,
                 GridBagConstraints.REMAINDER,
                 GridBagConstraints.RELATIVE, 1);
        boxNewRefresh.setState(prop.getBooleanProperty("refreshNew"));
        addField(panel, gbl, gbc,
                 boxNewRefresh, GridBagConstraints.REMAINDER, 1, 0);

        boxAutoRefresh.setState(prop.getBooleanProperty("autoRefresh"));
        boxAutoRefresh.addItemListener(this);
        addField(panel, gbl, gbc, boxAutoRefresh, 1, 1, 0);
        textAutoRefresh.setText(
                 String.valueOf(prop.getIntProperty("refreshInterval")));
        textAutoRefresh.setEnabled(boxAutoRefresh.getState());
        addField(panel, gbl, gbc, textAutoRefresh,
                 GridBagConstraints.RELATIVE,
                 GridBagConstraints.RELATIVE, 0);
        addField(panel, gbl, gbc, labelAutoRefresh,
                 GridBagConstraints.REMAINDER,
                 GridBagConstraints.RELATIVE, 0);

        boxDoSort.setState(prop.getBooleanProperty("doSort"));
        boxDoSort.addItemListener(this);
        boxNewOnTop.setState(prop.getBooleanProperty("changedFirst"));
        boxNewOnTop.setEnabled(boxDoSort.getState());
        boxIgnoreCaseInSort.setState(prop.getBooleanProperty("ignoreCaseInSort"));
        boxIgnoreCaseInSort.setEnabled(boxDoSort.getState());
        addField(panel, gbl, gbc, boxDoSort, 1, 1, 0);
        addField(panel, gbl, gbc, boxNewOnTop,
                 GridBagConstraints.RELATIVE,
                 GridBagConstraints.RELATIVE, 0);
        addField(panel, gbl, gbc, boxIgnoreCaseInSort,
                 GridBagConstraints.RELATIVE,
                 GridBagConstraints.RELATIVE, 0);
        boxUnderline.setState(prop.getBooleanProperty("underline"));
        addField(panel, gbl, gbc,
                 boxUnderline, GridBagConstraints.REMAINDER, 1, 0);

        boxUseProxy.setState(prop.getBooleanProperty("useProxy"));
        boxUseProxy.addItemListener(this);
        textProxyHost.setText(prop.getProperty("proxy.host"));
        textProxyPort.setText(
                          String.valueOf(prop.getIntProperty("proxy.port")));
        textProxyHost.setEnabled(boxUseProxy.getState());
        textProxyPort.setEnabled(boxUseProxy.getState());
        proxyPanel = new Panel(new FlowLayout());
        proxyPanel.add(labelProxyHost);
        proxyPanel.add(textProxyHost);
        proxyPanel.add(labelProxyPort);
        proxyPanel.add(textProxyPort);
        addField(panel, gbl, gbc, boxUseProxy, 1, 1, 0);
        addField(panel, gbl, gbc, proxyPanel,
                 GridBagConstraints.REMAINDER,
                 GridBagConstraints.RELATIVE, 0);

        return panel;
    }

    private void doOk(Frame parent) {
        int    interval, port;
        String browser;

        /* check that the given interval is ok. */
        try {
            interval = Integer.parseInt(textAutoRefresh.getText());
        } catch (NumberFormatException e) {
            interval = 0;
        }
        if (interval < 1)
            interval = 1;

        try {
            port = Integer.parseInt(textProxyPort.getText());
        } catch (NumberFormatException e) {
            port = 3128; /* default for Squid cache */
        }

        /* check that browser line contains %u, but only if changed from
         * the original value. */
        browser = textBrowser.getText();
        if (!browser.equals(prop.getProperty("browser")))
            if (browser.indexOf("%u") < 0 && browser.indexOf("%U") < 0)
                MessageBox.information(parent,
                                       "Warning: You gave a browser command "
                                       + "without URL parameter %u.");

        /* update the properties from the dialog contents. */
        prop.setProperty("pagesFile", textStatusFile.getText());
        prop.setProperty("browser", browser);
        prop.setProperty("alternatebrowsercommand",textAlternateBrowser.getText());
        prop.setBooleanProperty("refreshNew", boxNewRefresh.getState());
        prop.setBooleanProperty("autoRefresh", boxAutoRefresh.getState());
        prop.setIntProperty("refreshInterval", interval);
        prop.setBooleanProperty("doSort", boxDoSort.getState());
        prop.setBooleanProperty("changedFirst", boxNewOnTop.getState());
        prop.setBooleanProperty("ignoreCaseInSort", boxIgnoreCaseInSort.getState());
        prop.setBooleanProperty("underline", boxUnderline.getState());
        prop.setBooleanProperty("useProxy", boxUseProxy.getState());
        prop.setProperty("proxy.host", textProxyHost.getText());
        prop.setIntProperty("proxy.port", port);

        try {
            prop.saveProperties();
        } catch (IOException e) {
            MessageBox.error(parent,
                             "Unable to save properties: " + e.getMessage());
        }
        okPressed = true;
    }

    private void doCancel(Frame parent) {
        okPressed = false;
    }

/*-----------------------------------------------------------------------+
 |  PUBLIC INTERFACE                                                     |
 +----------------------------------------------------------------------*/
    public PropertyEditor(Frame parent, PropertyProvider prop) {
        ModalButtonDialog dlg;
        int               code;

        this.prop = prop;

        dlg = new ModalButtonDialog(
                      parent, "Preferences",
                      setupPanel(),
                      ModalButtonDialog.OK | ModalButtonDialog.CANCEL,
                      ModalButtonDialog.OK);

        code = dlg.showAndGetButtonCode();
        if (code == ModalButtonDialog.OK)
            doOk(parent);
        else
            doCancel(parent);
    }

    public boolean isOkPressed() {
        return okPressed;
    }

    /* ItemListener *****************************************************/
    public void itemStateChanged(ItemEvent e) {
        Object src;

        src = e.getSource();
        if (src == boxDoSort) {
            boxNewOnTop.setEnabled(boxDoSort.getState());
            boxIgnoreCaseInSort.setEnabled(boxDoSort.getState());
        } else if (src == boxUseProxy) {
            textProxyHost.setEnabled(boxUseProxy.getState());
            textProxyPort.setEnabled(boxUseProxy.getState());
        } else if (src == boxAutoRefresh)
            textAutoRefresh.setEnabled(boxAutoRefresh.getState());
    }
}
